/*
 *  libfb - FreeBASIC's runtime library
 *	Copyright (C) 2004-2005 Andre V. T. Vicentini (av1ctor@yahoo.com.br) and others.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * str_ftoa.c -- float to string, internal usage
 *
 * chng: dec/2005 written [v1ctor]
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "fb.h"

#ifdef TARGET_WIN32
# define snprintf _snprintf
#endif

/*:::::*/
char *fb_hFloat2Str( double val, char *buffer, int digits, int mask )
{
	int len;
	char *p;
	char fmtstr[16];

	if( mask & FB_F2A_ADDBLANK )
		p = &buffer[1];
	else
		p = buffer;

	/* no exponent? (if exp is too big, that won't matter) */
	if( (mask & FB_F2A_NOEXP) > 0 )
	{
		sprintf( fmtstr, "%%.%df", digits );
		if( snprintf( p, digits+1+3+1+1+1, fmtstr, val ) <= 0 )
			return NULL;
	}
	else
	{
		/* why not use always %f? because it will print dizima's,
		   123467.9 will become 123467.89999999999 for example */
#ifdef TARGET_WIN32
		_gcvt( val, digits, p );
#else
		sprintf( fmtstr, "%%.%dg", digits );
		if( snprintf( p, digits+1+3+1+1+1, fmtstr, val ) <= 0 )
			return NULL;
#endif
	}

	len = strlen( p );

	if( len > 0 )
	{
		if( (mask & FB_F2A_NOEXP) > 0 )
			/* skip the zeros at end */
			while( p[len-1] == '0' )
				p[--len] = '\0';

		/* skip the dot at end if any */
		if( len > 0 )
			if( p[len-1] == '.' )
				p[len-1] = '\0';
	}

	/* */
	if( (mask & FB_F2A_ADDBLANK) > 0 )
	{
		if( p[0] != '-' )
		{
			buffer[0] = ' ';
			return &buffer[0];
		}
		else
			return p;
	}
	else
		return p;

}

